#
# Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

from netq_agent.parsers.nvos.redis_db.client import NvosDBRender
from netq_agent.parsers.sonic.redis_db.sonic_db import SonicDB, SonicTable
from netq_lib.orm.redisdb.models import IbEvent


class EventsIbHandler(NvosDBRender):
    MODEL = IbEvent

    def __init__(self):
        super(EventsIbHandler, self).__init__()

    def db_sources(self):
        return {
            (SonicDB.EVENT_DB, SonicTable.EVENT_DB_EVENT):
                self.handle_event
        }

    def get_event_id(self, event):
        return int(event.get('id', 0))  # Return 0 if 'id' key is not found

    def handle_event(self, table, raw, data):
        self._logger.debug(f"Handling new event for table: {table} raw: {raw} data: {data}")
        ib_event_obj = IbEvent()
        if data == 'del':
            ib_event_obj.event_id = raw
            ib_event_obj.deleted = True
            ib_event_obj.time_created = 0
        else:
            ib_event_obj.event_id = data.get('id')
            ib_event_obj.resource = data.get('resource')
            ib_event_obj.severity = data.get('severity')
            ib_event_obj.text = data.get('text')
            ib_event_obj.time_created = data.get('time-created')
            ib_event_obj.type_id = data.get('type-id')
        ib_event_obj.save()

    def process_fullstate(self, data):
        events_dict = data.get(SonicDB.EVENT_DB, "").get(SonicTable.EVENT_DB_EVENT, "")
        sorted_events = sorted(events_dict.values(), key=self.get_event_id)
        for event in sorted_events:
            ib_event_obj = IbEvent()
            ib_event_obj.event_id = event.get('id')
            ib_event_obj.resource = event.get('resource')
            ib_event_obj.severity = event.get('severity')
            ib_event_obj.text = event.get('text')
            ib_event_obj.time_created = event.get('time-created')
            ib_event_obj.type_id = event.get('type-id')
            ib_event_obj.save()


def process_events(*args, **kwargs):
    logger = kwargs.get('logger')
    logger.info('process_events invoked')
    render.process(logger)
    subscribe_manager = kwargs.get('subscribe_manager')
    subscribe_manager.register_callback(render)


render = EventsIbHandler()
